from .TCPserver import TCPMultiplayerServer
from .client import MultiplayerClient
print("This library is currently undergoing a major rewrite.")
print("Not all features will work as intended.")
print("pymultiplayer by iamdeedz")
print("https://www.github.com/iamdeedz/pymultiplayer")
